// Copyright (C) 2018-2022 Intel Corporation
// SPDX-License-Identifier: Apache-2.0
//


#include "behavior/ov_infer_request/perf_counters.hpp"
#include "openvino/runtime/profiling_info.hpp"

namespace ov {
namespace test {
namespace behavior {

std::string OVInferRequestPerfCountersTest::getTestCaseName(const testing::TestParamInfo<InferRequestParams>& obj) {
    return OVInferRequestTests::getTestCaseName(obj);
}

void OVInferRequestPerfCountersTest::SetUp() {
    SKIP_IF_CURRENT_TEST_IS_DISABLED()
    std::tie(targetDevice, configuration) = this->GetParam();
    function = ngraph::builder::subgraph::makeConvPoolRelu();
    configuration.insert({ InferenceEngine::PluginConfigParams::KEY_PERF_COUNT, InferenceEngine::PluginConfigParams::YES });
    execNet = core->compile_model(function, targetDevice, configuration);
    req = execNet.create_infer_request();
}

TEST_P(OVInferRequestPerfCountersTest, NotEmptyAfterAsyncInfer) {
    OV_ASSERT_NO_THROW(req.start_async());
    OV_ASSERT_NO_THROW(req.wait());
    std::vector<runtime::ProfilingInfo> perf;
    OV_ASSERT_NO_THROW(perf = req.get_profiling_info());
    ASSERT_FALSE(perf.empty());
}

TEST_P(OVInferRequestPerfCountersTest, NotEmptyAfterSyncInfer) {
    OV_ASSERT_NO_THROW(req.infer());
    std::vector<runtime::ProfilingInfo> perf;
    OV_ASSERT_NO_THROW(perf = req.get_profiling_info());
    ASSERT_FALSE(perf.empty());
}
}  // namespace behavior
}  // namespace test
}  // namespace ov